// $Id: HOPSPACK_EvaluatorDefault.cpp 149 2009-11-12 02:40:41Z tplante $
// $URL: https://software.sandia.gov/svn/public/hopspack/trunk/src/src-evaluator/HOPSPACK_EvaluatorDefault.cpp $

//@HEADER
// ************************************************************************
// 
//         HOPSPACK: Hybrid Optimization Parallel Search Package
//                 Copyright 2009 Sandia Corporation
// 
// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
// the U.S. Government retains certain rights in this software.
// 
// This file is part of HOPSPACK.
//
// HOPSPACK is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//  
// You should have received a copy of the GNU Lesser General Public
// License along with this library.  If not, see http://www.gnu.org/licenses/.
// 
// Questions? Contact Tammy Kolda (tgkolda@sandia.gov)
//                 or Todd Plantenga (tplante@sandia.gov) 
// 
// ************************************************************************
//@HEADER

/*!
  @file HOPSPACK_EvaluatorDefault.cpp
  @brief Implement HOPSPACK::EvaluatorDefault.
*/

#include "HOPSPACK_common.hpp"
#include "HOPSPACK_EvaluatorDefault.hpp"
#include "HOPSPACK_SystemCall.hpp"

namespace HOPSPACK
{


//----------------------------------------------------------------------
//  Static data
//----------------------------------------------------------------------

const char  EvaluatorDefault::sEVAL_PARAM_ERROR[]
                = "ERROR IN EVALUATOR PARAMETERS";


//----------------------------------------------------------------------
//  Construct an instance based on user parameters
//----------------------------------------------------------------------
EvaluatorDefault *  EvaluatorDefault::newInstance
                        (const ParameterList &  cEvalParams)
{
    //---- NEED A TYPE PARAMETER TO CHOOSE THE CONCRETE IMPLEMENTATION.
    string  sType = cEvalParams.getParameter ("Evaluator Type", "System Call");

    EvaluatorDefault *  pResult = NULL;

    if (sType == "System Call")
    {
        try
        {
            pResult = (EvaluatorDefault *) new SystemCall (cEvalParams);
        }
        catch (const char * const)     //-- EXPECTING sEVAL_PARAM_ERROR
        {
            pResult = NULL;
        }
    }
    else
    {
        //---- CANNOT FIGURE OUT THE TYPE OF EVALUATOR.
        cerr << "ERROR: The value '" << sType
             << "' in parameter 'Evaluator Type' is not recognized." << endl;
        cerr << "  Please change parameter 'Evaluator Type' in sublist "
             << "'Evaluator'." << endl;
    }

    return pResult;
}


//----------------------------------------------------------------------
//  Default constructor for Base class does nothing
//----------------------------------------------------------------------
EvaluatorDefault::EvaluatorDefault (void)
{
    return;
}


//----------------------------------------------------------------------
//  Base class needs a destructor even though it's declared pure virtual
//----------------------------------------------------------------------
EvaluatorDefault::~EvaluatorDefault (void)
{
    return;
}


}     //-- namespace HOPSPACK
